/** @file
  Virtio Network Device specific type and macro definitions corresponding to
  the virtio specifications.

  Copyright (C) 2013-2016, Red Hat, Inc.

  This program and the accompanying materials are licensed and made available
  under the terms and conditions of the BSD License which accompanies this
  distribution. The full text of the license may be found at
  http://opensource.org/licenses/bsd-license.php

  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS, WITHOUT
  WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.

**/

#ifndef _VIRTIO_NET_H_
#define _VIRTIO_NET_H_

#include <IndustryStandard/Virtio10Net.h>

#endif // _VIRTIO_NET_H_
