;; @file
;
;@copyright
; Copyright (c) 2015 - 2016, Intel Corporation. All rights reserved.<BR>
; This program and the accompanying materials
; are licensed and made available under the terms and conditions of the BSD License
; which accompanies this distribution.  The full text of the license may be found at
; http://opensource.org/licenses/bsd-license.php.
;
; THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
; WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
;;

MSR_IA32_PLATFORM_ID        equ     000000017h
MSR_IA32_BIOS_UPDT_TRIG     equ     000000079h
MSR_IA32_BIOS_SIGN_ID       equ     00000008bh